package ink.wgink.module.examine.controller.test;

import com.cm.study.enums.QuestionTypeEnum;
import com.cm.study.pojo.dtos.question.QuestionDTO;
import com.cm.study.pojo.vos.question.QuestionVO;
import com.cm.study.service.question.IQuestionService;
import ink.wgink.annotation.CheckRequestBodyAnnotation;
import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.common.component.SecurityComponent;
import ink.wgink.exceptions.ParamsException;
import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.interfaces.consts.IFileConstant;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.*;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @ClassName: TestQuestionController
 * @Description: 试题管理
 * @Author: WenG
 * @Date: 2020-09-09 21:00
 * @Version: 1.0
 **/
@Api(tags = ISystemConstant.API_TAGS_SYSTEM_PREFIX + "试题管理接口")
@RestController
@RequestMapping(ISystemConstant.API_PREFIX + "/test/question")
public class TestQuestionController extends DefaultBaseController {

    @Autowired
    private IQuestionService questionService;
    @Autowired
    private SecurityComponent securityComponent;

    @ApiOperation(value = "新增试题管理", notes = "新增试题管理接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("save")
    @CheckRequestBodyAnnotation
    public SuccessResult saveQuestion(@RequestBody QuestionVO questionVO) throws Exception {
        boolean hanOption = StringUtils.equals(questionVO.getType(), QuestionTypeEnum.CHOICE.getValue())
                || StringUtils.equals(questionVO.getType(), QuestionTypeEnum.TURE_OR_FALSE.getValue());
        if (hanOption && questionVO.getQuestionOptions().isEmpty()) {
            throw new ParamsException("选择或判断选项不能为空");
        }
        return questionService.saveQuestion(questionVO);
    }

    @ApiOperation(value = "删除试题管理(id列表)", notes = "删除试题管理(id列表)接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "ID列表，用下划线分隔", paramType = "path", example = "1_2_3")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @DeleteMapping("removequestion/{ids}")
    public SuccessResult removeQuestion(@PathVariable("ids") String ids) throws RemoveException {
        questionService.removeQuestion(Arrays.asList(ids.split("\\_")));
        return new SuccessResult();
    }

    @ApiOperation(value = "修改试题管理", notes = "修改试题管理接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "questionId", value = "试题管理ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("updatequestion/{questionId}")
    @CheckRequestBodyAnnotation
    public SuccessResult updateQuestion(@PathVariable("questionId") String questionId, @RequestBody QuestionVO questionVO) throws Exception {
        boolean hanOption = StringUtils.equals(questionVO.getType(), QuestionTypeEnum.CHOICE.getValue())
                || StringUtils.equals(questionVO.getType(), QuestionTypeEnum.TURE_OR_FALSE.getValue());
        if (hanOption && questionVO.getQuestionOptions().isEmpty()) {
            throw new ParamsException("选择或判断选项不能为空");
        }
        return questionService.updateQuestion(questionId, questionVO);
    }

    @ApiOperation(value = "试题管理详情(通过ID)", notes = "试题管理详情(通过ID)接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "questionId", value = "试题管理ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("getquestionbyid/{questionId}")
    public QuestionDTO getQuestionById(@PathVariable("questionId") String questionId) throws SearchException {
        return questionService.getQuestionById(questionId);
    }

    @ApiOperation(value = "试题管理列表", notes = "试题管理列表接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listquestion")
    public List<QuestionDTO> listQuestion() throws SearchException {
        Map<String, Object> params = requestParams();
        return questionService.listQuestion(params);
    }

    @ApiOperation(value = "试题管理分页列表", notes = "试题管理分页列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpagequestion")
    public SuccessResultList<List<QuestionDTO>> listPageQuestion(ListPage page) throws SearchException {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return questionService.listPageQuestion(page);
    }

    @ApiOperation(value = "试题管理分页（管理员）列表", notes = "试题管理分页（管理员）列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpageadmin")
    public SuccessResultList<List<QuestionDTO>> listPageAdmin(ListPage page) throws SearchException {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return questionService.listPageAdmin(page);
    }

    @ApiOperation(value = "试题管理分页（老师）列表", notes = "试题管理分页（老师）列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpageteacher")
    public SuccessResultList<List<QuestionDTO>> listPageTeacher(ListPage page) throws SearchException {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return questionService.listPageTeacher(page);
    }

    @ApiOperation(value = "试题管理统计", notes = "试题管理统计接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("countquestion")
    SuccessResultData<Integer> countQuestion() throws SearchException {
        Map<String, Object> params = requestParams();
        return questionService.countQuestion(params);
    }

    @ApiOperation(value = "试题管理（排除）分页列表", notes = "试题管理（排除）分页列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "classInfoId", value = "班级ID", paramType = "path"),
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpageexcept/{classInfoId}")
    public SuccessResultList<List<QuestionDTO>> listPageExcept(@PathVariable("classInfoId") String classInfoId, ListPage page) {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return questionService.listPageExcept(classInfoId, page);
    }

    @ApiOperation(value = "试题管理（排除）分页列表（管理员）", notes = "试题管理（排除）分页列表（管理员）接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "classInfoId", value = "当前页码", paramType = "path"),
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpageexceptadmin/{classInfoId}")
    public SuccessResultList<List<QuestionDTO>> listPageExceptAdmin(@PathVariable("classInfoId") String classInfoId, ListPage page) {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return questionService.listPageExceptAdmin(classInfoId, page);
    }

    @ApiOperation(value = "试题管理（排除）分页列表（老师）", notes = "试题管理（排除）分页列表（老师）接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "classInfoId", value = "当前页码", paramType = "path"),
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpageexceptteacher/{classInfoId}")
    public SuccessResultList<List<QuestionDTO>> listPageExceptTeachers(@PathVariable("classInfoId") String classInfoId, ListPage page) {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return questionService.listPageExceptTeacher(classInfoId, page);
    }

    @ApiOperation(value = "试题导入Excel", notes = "试题导入Excel接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "excel", value = "文件名称", paramType = "query"),
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("import-excel")
    public UploadExcelResultDTO importExcel(MultipartFile excel) throws IOException {
        if (Objects.isNull(excel)) {
            throw new ParamsException("Excel不能为空");
        }
        if (!excel.getOriginalFilename().endsWith(IFileConstant.EXCEL_SUFFIX_XLS) &&
                !excel.getOriginalFilename().endsWith(IFileConstant.EXCEL_SUFFIX_XLSX)) {
            throw new ParamsException("文件格式为Excel");
        }
        return questionService.importExcel(excel);
    }

}