package ink.wgink.module.examine.service.test;

import com.cm.study.enums.QuestionChoiceTypeEnum;
import com.cm.study.enums.QuestionTypeEnum;
import com.cm.study.pojo.dtos.classquestion.ClassQuestionDTO;
import com.cm.study.pojo.dtos.question.QuestionDTO;
import com.cm.study.pojo.vos.question.TestQuestionVO;
import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultData;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.pojo.result.UploadExcelResultDTO;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: IQuestionService
 * @Description: 试题管理
 * @Author: WenG
 * @Date: 2020-09-09 21:00
 * @Version: 1.0
 **/
public interface ITestQuestionService {

    /**
     * 新增试题管理
     *
     * @param questionVO
     * @return
     * @throws Exception
     */
    SuccessResult save(TestQuestionVO questionVO) throws Exception;

    /**
     * @return
     * @throws Exception
     */
    SuccessResult saveByToken(String token, TestQuestionVO questionVO) throws Exception;

    /**
     * 新增试题管理
     *
     * @param questionVO
     * @return questionId
     * @throws Exception
     */
    String saveReturnId(TestQuestionVO questionVO) throws Exception;

    /**
     * 新增试题管理(APP)
     *
     * @param token
     * @param questionVO
     * @return questionId
     * @throws Exception
     */
    String saveByTokenReturnId(String token, TestQuestionVO questionVO) throws Exception;

    /**
     * 删除试题管理
     *
     * @param ids
     * @return
     * @throws RemoveException
     */
    void remove(List<String> ids) throws RemoveException;

    /**
     * 删除试题管理（物理删除）
     *
     * @param ids
     * @throws RemoveException
     */
    void delete(String ids) throws RemoveException;

    /**
     * 删除试题管理(APP)
     *
     * @param token
     * @param ids
     * @return
     * @throws RemoveException
     */
    void removeByToken(String token, List<String> ids) throws RemoveException;

    /**
     * 修改试题管理
     *
     * @param questionId
     * @param questionVO
     * @return
     * @throws Exception
     */
    SuccessResult update(String questionId, TestQuestionVO questionVO) throws Exception;

    /**
     * 修改试题管理(APP)
     *
     * @param token
     * @param questionId
     * @param questionVO
     * @return
     * @throws Exception
     */
    SuccessResult updateByToken(String token, String questionId, TestQuestionVO questionVO) throws Exception;

    /**
     * 试题管理详情(通过ID)
     *
     * @param questionId
     * @return
     * @throws SearchException
     */
    QuestionDTO getById(String questionId) throws SearchException;

    /**
     * 试题管理列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<QuestionDTO> list(Map<String, Object> params) throws SearchException;

    /**
     * 试题列表
     *
     * @param questionIds
     * @return
     */
    List<QuestionDTO> listByIds(List<String> questionIds);

    /**
     * 试题管理分页列表
     *
     * @param page
     * @return
     * @throws SearchException
     */
    SuccessResultList<List<QuestionDTO>> listPage(ListPage page) throws SearchException;

    /**
     * 试题管理统计
     *
     * @param params
     * @return
     * @throws SearchException
     */
    Integer countNumberQuestion(Map<String, Object> params) throws SearchException;

    /**
     * 试题导入Excel
     * @param excel
     * @return
     * @throws IOException
     */
    UploadExcelResultDTO importExcel(MultipartFile excel) throws IOException;

    /**
     * 更新自定义问题类型
     *
     * @param updateType
     * @param questionTypeId
     * @param questionIds
     */
    void updateCustomQuestionType(String questionTypeId, List<String> questionIds);

    /**
     * 删除自定义问题类型
     *
     * @param updateType
     * @param questionTypeId
     * @param questionIds
     */
    void removeCustomQuestionType(String questionTypeId, List<String> questionIds);
}