package ink.wgink.module.examine.service.test.impl;

import com.alibaba.excel.EasyExcel;
import com.cm.study.dao.question.IQuestionDao;
import com.cm.study.enums.QuestionChoiceTypeEnum;
import com.cm.study.enums.QuestionTypeEnum;
import com.cm.study.listener.question.QuestionExcel;
import com.cm.study.listener.question.QuestionExcelError;
import com.cm.study.listener.question.QuestionExcelListener;
import com.cm.study.pojo.dtos.classquestion.ClassQuestionDTO;
import com.cm.study.pojo.dtos.question.QuestionDTO;
import com.cm.study.pojo.dtos.questionoptions.QuestionOptionsDTO;
import com.cm.study.pojo.dtos.questiontype.QuestionTypeDTO;
import com.cm.study.pojo.dtos.schoolteacher.SchoolTeacherDTO;
import com.cm.study.pojo.pos.question.QuestionPO;
import com.cm.study.pojo.vos.question.QuestionVO;
import com.cm.study.pojo.vos.questionoptions.QuestionOptionsVO;
import com.cm.study.service.BaseService;
import com.cm.study.service.classquestion.IClassQuestionService;
import com.cm.study.service.examinationpaperquestion.IExaminationPaperQuestionService;
import com.cm.study.service.question.IQuestionService;
import com.cm.study.service.questionoptions.IQuestionOptionsService;
import com.cm.study.service.questiontype.IQuestionTypeService;
import com.cm.study.service.schoolteacher.ISchoolTeacherService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.module.dictionary.pojo.dtos.DataDTO;
import ink.wgink.module.dictionary.service.IDataService;
import ink.wgink.module.examine.service.test.ITestQuestionService;
import ink.wgink.module.file.excel.error.AbstractErrorExcelHandler;
import ink.wgink.module.file.service.IFileService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultData;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.pojo.result.UploadExcelResultDTO;
import ink.wgink.util.UUIDUtil;
import ink.wgink.util.map.HashMapUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * @ClassName: QuestionServiceImpl
 * @Description: 试题管理
 * @Author: WenG
 * @Date: 2020-09-09 21:00
 * @Version: 1.0
 **/
@Service
public class TestQuestionServiceImpl extends DefaultBaseService implements ITestQuestionService {

    @Autowired
    private IQuestionDao questionDao;
    @Autowired
    private IQuestionOptionsService questionOptionsService;
    @Autowired
    private IClassQuestionService classQuestionService;
    @Autowired
    private ISchoolTeacherService schoolTeacherService;
    @Autowired
    private IDataService dataService;
    @Autowired
    private IFileService fileService;
    @Autowired
    private IQuestionTypeService questionTypeService;
    @Autowired
    private IExaminationPaperQuestionService examinationPaperQuestionService;

    @Override
    public SuccessResult saveQuestion(QuestionVO questionVO) throws Exception {
        saveQuestionInfo(null, questionVO);
        return new SuccessResult();
    }

    @Override
    public SuccessResult saveQuestionByToken(String token, QuestionVO questionVO) throws Exception {
        saveQuestionInfo(token, questionVO);
        return new SuccessResult();
    }

    @Override
    public String saveQuestionReturnId(QuestionVO questionVO) throws Exception {
        return saveQuestionInfo(null, questionVO);
    }

    @Override
    public String saveQuestionByTokenReturnId(String token, QuestionVO questionVO) throws Exception {
        return saveQuestionInfo(token, questionVO);
    }

    /**
     * 新增试题管理
     *
     * @param token
     * @param questionVO
     * @throws Exception
     */
    private String saveQuestionInfo(String token, QuestionVO questionVO) throws Exception {
        String questionId = saveQuestionInfoReturnId(token, questionVO);
        boolean hasOption = StringUtils.equals(questionVO.getType(), QuestionTypeEnum.CHOICE.getValue())
                || StringUtils.equals(questionVO.getType(), QuestionTypeEnum.TURE_OR_FALSE.getValue());
        if (!hasOption) {
            return questionId;
        }
        // 保存选项
        for (QuestionOptionsVO questionOptionsVO : questionVO.getQuestionOptions()) {
            questionOptionsVO.setQuestionId(questionId);
            questionOptionsService.saveQuestionOptionsByToken(token, questionOptionsVO);
        }
        return questionId;
    }

    /**
     * 新增试题管理
     *
     * @param token
     * @param questionVO
     * @return questionId
     * @throws Exception
     */
    private String saveQuestionInfoReturnId(String token, QuestionVO questionVO) throws Exception {
        questionVO.setSubject(questionVO.getSubject().replaceAll("\\s", ""));
        QuestionPO questionPO = getPO(questionVO.getSubject());
        if (questionPO != null) {
            throw new SearchException("题目已经存在");
        }
        String questionId = UUIDUtil.getUUID();
        Map<String, Object> params = HashMapUtil.beanToMap(questionVO);
        params.put("questionId", questionId);
        if (token != null) {
            setAppSaveInfo(token, params);
        } else {
            setSaveInfo(params);
        }
        questionDao.saveQuestion(params);
        return questionId;
    }

    @Override
    public void removeQuestion(List<String> ids) throws RemoveException {
        removeQuestionInfo(null, ids);
    }

    @Override
    public void removeQuestionByToken(String token, List<String> ids) throws RemoveException {
        removeQuestionInfo(token, ids);
    }

    /**
     * 删除试题管理
     *
     * @param token
     * @param ids
     */
    private void removeQuestionInfo(String token, List<String> ids) {
        checkQuestionUse(ids);
        Map<String, Object> params = getHashMap(3);
        params.put("questionIds", ids);
        if (token != null) {
            setAppUpdateInfo(token, params);
        } else {
            setUpdateInfo(params);
        }
        questionDao.removeQuestion(params);
    }

    @Override
    public void deleteQuestion(String ids) throws RemoveException {
        Map<String, Object> params = getHashMap(3);
        params.put("questionIds", Arrays.asList(ids.split("_")));
        questionDao.deleteQuestion(params);
    }

    @Override
    public SuccessResult updateQuestion(String questionId, QuestionVO questionVO) throws Exception {
        updateQuestionInfo(null, questionId, questionVO);
        return new SuccessResult();
    }

    @Override
    public SuccessResult updateQuestionByToken(String token, String questionId, QuestionVO questionVO) throws Exception {
        updateQuestionInfo(token, questionId, questionVO);
        return new SuccessResult();
    }

    /**
     * 修改试题管理
     *
     * @param token
     * @param questionId
     * @param questionVO
     */
    private void updateQuestionInfo(String token, String questionId, QuestionVO questionVO) throws Exception {
        checkQuestionUse(Arrays.asList(questionId));

        Map<String, Object> params = HashMapUtil.beanToMap(questionVO);
        params.put("questionId", questionId);
        if (token != null) {
            setAppUpdateInfo(token, params);
        } else {
            setUpdateInfo(params);
        }
        questionDao.updateQuestion(params);
        // 判断是否更新选项
        boolean hasOption = StringUtils.equals(questionVO.getType(), QuestionTypeEnum.CHOICE.getValue())
                || StringUtils.equals(questionVO.getType(), QuestionTypeEnum.TURE_OR_FALSE.getValue());
        if (!hasOption) {
            return;
        }
        // 删除原有选项
        questionOptionsService.deleteQuestionOptionsByQuestionId(questionId);
        // 重新添加新选项
        for (QuestionOptionsVO questionOptionsVO : questionVO.getQuestionOptions()) {
            questionOptionsVO.setQuestionId(questionId);
            questionOptionsService.saveQuestionOptionsByToken(token, questionOptionsVO);
        }
    }

    /**
     * 判断试题是否已经使用，已经使用的试题无法修改
     *
     * @param questionIds
     */
    private void checkQuestionUse(List<String> questionIds) {
        List<String> useQuestionIds = examinationPaperQuestionService.listQuestionIds(questionIds);
        if (!useQuestionIds.isEmpty()) {
            throw new SearchException("无法处理，试题已经被使用");
        }
    }

    private QuestionPO getPO(Map<String, Object> params) {
        return questionDao.getPO(params);
    }

    public QuestionPO getPO(String subject) {
        Map<String, Object> params = getHashMap(2);
        params.put("subject", subject);
        return getPO(params);
    }

    @Override
    public QuestionDTO getQuestionById(String questionId) throws SearchException {
        Map<String, Object> params = super.getHashMap(1);
        params.put("questionId", questionId);
        QuestionDTO questionDTO = questionDao.getQuestion(params);
        boolean hasOption = StringUtils.equals(questionDTO.getType(), QuestionTypeEnum.CHOICE.getValue())
                || StringUtils.equals(questionDTO.getType(), QuestionTypeEnum.TURE_OR_FALSE.getValue());
        if (hasOption) {
            List<QuestionOptionsDTO> questionOptionsDTOs = questionOptionsService.listQuestionOptionsByQuestionId(questionId);
            questionDTO.setQuestionOptions(questionOptionsDTOs);
        }
        if (!StringUtils.isBlank(questionDTO.getCustomQuestionType())) {
            // 加载自定义问题类型
            List<String> questionTypeIds = Arrays.asList(questionDTO.getCustomQuestionType().split(","));
            List<QuestionTypeDTO> questionTypeDTOs = questionTypeService.list(questionTypeIds);
            questionDTO.setCustomQuestionTypeName(getCustomQuestionTypeNames(questionDTO.getCustomQuestionType(), questionTypeDTOs));
        }
        return questionDTO;
    }


    @Override
    public List<QuestionDTO> listQuestion(Map<String, Object> params) throws SearchException {
        if (params.get("customQuestionType") != null && !StringUtils.isBlank(params.get("customQuestionType").toString())) {
            params.put("customQuestionTypes", Arrays.asList(params.get("customQuestionType").toString().split(",")));
            params.remove("customQuestionType");
        }
        List<QuestionDTO> questionDTOs = questionDao.listQuestion(params);
        // 试题类型
        List<DataDTO> dataDTOs = dataService.listByParentId(QUESTION_TYPE_ID);

        for (QuestionDTO questionDTO : questionDTOs) {
            String questionTypeName = questionDTO.getQuestionType();
            if (StringUtils.isBlank(questionTypeName)) {
                continue;
            }
            for (DataDTO dataDTO : dataDTOs) {
                questionTypeName = questionTypeName.replace(dataDTO.getDataId(), dataDTO.getDataName());
            }
            questionDTO.setQuestionTypeName(questionTypeName);
        }
        // 自定义试题类型
        Set<String> customQuestionTypeSet = new HashSet<>();
        for (QuestionDTO questionDTO : questionDTOs) {
            String customQuestionTypes = questionDTO.getCustomQuestionType();
            if (StringUtils.isBlank(customQuestionTypes)) {
                continue;
            }
            String[] customQuestionTypeArray = customQuestionTypes.split(",");
            for (String customQuestionType : customQuestionTypeArray) {
                if (StringUtils.isBlank(customQuestionType)) {
                    continue;
                }
                customQuestionTypeSet.add(customQuestionType);
            }
        }
        if (!customQuestionTypeSet.isEmpty()) {
            String[] customQuestionTypeArray = new String[customQuestionTypeSet.size()];
            List<String> questionTypeIds = Arrays.asList(customQuestionTypeSet.toArray(customQuestionTypeArray));
            List<QuestionTypeDTO> questionTypeDTOs = questionTypeService.list(questionTypeIds);
            for (QuestionDTO questionDTO : questionDTOs) {
                String customQuestionTypes = questionDTO.getCustomQuestionType();
                if (StringUtils.isBlank(customQuestionTypes)) {
                    continue;
                }
                questionDTO.setCustomQuestionTypeName(getCustomQuestionTypeNames(customQuestionTypes, questionTypeDTOs));
            }
        }
        // 设置试题使用状态
        setQuestionUsed(questionDTOs);
        return questionDTOs;
    }

    @Override
    public List<QuestionDTO> listByIds(List<String> questionIds) {
        Map<String, Object> params = getHashMap(2);
        params.put("questionIds", questionIds);
        return listQuestion(params);
    }

    /**
     * 设置试题是否被使用
     *
     * @param questionDTOs
     */
    private void setQuestionUsed(List<QuestionDTO> questionDTOs) {
        if (questionDTOs.isEmpty()) {
            return;
        }
        Set<String> questionIdSet = new HashSet<>();
        questionDTOs.forEach(questionDTO -> questionIdSet.add(questionDTO.getQuestionId()));
        List<String> useQuestionIds = examinationPaperQuestionService.listQuestionIds(new ArrayList<>(questionIdSet));
        questionDTOs.forEach(questionDTO -> {
            for (String questionId : useQuestionIds) {
                if (StringUtils.equals(questionDTO.getQuestionId(), questionId)) {
                    questionDTO.setUsed(true);
                }
            }
        });
    }

    /**
     * 处理自定义问题类型名称
     *
     * @param customQuestionTypes
     * @param questionTypeDTOs
     * @return
     */
    private String getCustomQuestionTypeNames(String customQuestionTypes, List<QuestionTypeDTO> questionTypeDTOs) {
        String customQuestionTypeNames = "";
        for (QuestionTypeDTO questionTypeDTO : questionTypeDTOs) {
            if (!customQuestionTypes.contains(questionTypeDTO.getQuestionTypeId())) {
                continue;
            }
            if (customQuestionTypeNames.length() > 0) {
                customQuestionTypeNames += ",";
            }
            customQuestionTypeNames += questionTypeDTO.getTypeName();
        }
        return customQuestionTypeNames;
    }

    @Override
    public SuccessResultList<List<QuestionDTO>> listPageQuestion(ListPage page) throws SearchException {
        PageHelper.startPage(page.getPage(), page.getRows());
        List<QuestionDTO> questionDTOs = listQuestion(page.getParams());
        PageInfo<QuestionDTO> pageInfo = new PageInfo<>(questionDTOs);
        return new SuccessResultList<>(questionDTOs, pageInfo.getPageNum(), pageInfo.getTotal());
    }

    @Override
    public SuccessResultList<List<QuestionDTO>> listPageAdmin(ListPage page) throws SearchException {
        List<SchoolTeacherDTO> schoolTeacherDTOs = schoolTeacherService.listSimpleBySchoolId(getAdminSchoolId());
        List<String> creators = new ArrayList<>();
        creators.add(securityComponent.getCurrentUser().getUserId());
        for (SchoolTeacherDTO schoolTeacherDTO : schoolTeacherDTOs) {
            creators.add(schoolTeacherDTO.getUserId());
        }
        page.getParams().put("creators", creators);
        return listPageQuestion(page);
    }

    @Override
    public SuccessResultList<List<QuestionDTO>> listPageTeacher(ListPage page) throws SearchException {
        page.getParams().put("creator", securityComponent.getCurrentUser().getUserId());
        return listPageQuestion(page);
    }

    @Override
    public Integer countNumberQuestion(Map<String, Object> params) throws SearchException {
        Integer count = questionDao.countQuestion(params);
        return count == null ? 0 : count;
    }

    @Override
    public SuccessResultData<Integer> countQuestion(Map<String, Object> params) throws SearchException {
        return new SuccessResultData<>(countNumberQuestion(params));
    }

    @Override
    public List<String> listRandomQuestionIdByRandomCountAndTypeAndChoiceType(int randomCount, QuestionTypeEnum type, QuestionChoiceTypeEnum choiceType) throws SearchException {
        Map<String, Object> params = getHashMap(4);
        params.put("type", type.getValue());
        params.put("choiceType", choiceType == null ? null : choiceType.getValue());
        List<String> questionIds = questionDao.listQuestionIds(params);
        // 试题数小于随机总数，返回全部试题ID
        if (questionIds.size() <= randomCount) {
            return questionIds;
        }
        Random random = new Random(System.currentTimeMillis());
        Map<Integer, String> randomMap = new HashMap<>(randomCount * 2);
        while (randomMap.size() < randomCount) {
            int randomNumber = random.nextInt(questionIds.size());
            randomMap.put(randomNumber, questionIds.get(randomNumber));
        }
        List<String> randomQuestionIds = new ArrayList<>();
        for (Map.Entry<Integer, String> kv : randomMap.entrySet()) {
            randomQuestionIds.add(kv.getValue());
        }
        return randomQuestionIds;
    }

    @Override
    public List<QuestionDTO> listQuestionByExaminationPagerId(String examinationPaperId) throws SearchException {
        return questionDao.listQuestionByExaminationPagerId(examinationPaperId);
    }

    @Override
    public SuccessResultList<List<QuestionDTO>> listPageExcept(ListPage page) {
        return null;
    }

    @Override
    public SuccessResultList<List<QuestionDTO>> listPageExcept(String classInfoId, ListPage page) throws SearchException {
        List<ClassQuestionDTO> classQuestionDTOs = classQuestionService.listSimple(classInfoId);
        List<String> withoutQuestionIds = new ArrayList<>();
        for (ClassQuestionDTO classQuestionDTO : classQuestionDTOs) {
            withoutQuestionIds.add(classQuestionDTO.getQuestionId());
        }
        page.getParams().put("withoutQuestionIds", withoutQuestionIds);
        return listPageQuestion(page);
    }

    @Override
    public SuccessResultList<List<QuestionDTO>> listPageExceptAdmin(String classInfoId, ListPage page) throws SearchException {
        List<SchoolTeacherDTO> schoolTeacherDTOs = schoolTeacherService.listSimpleBySchoolId(getAdminSchoolId());
        // 学校列表列出超级管理员、当前管理员和全部学校老师的试题
        List<String> creators = new ArrayList<>();
        creators.add(securityComponent.getCurrentUser().getUserId());
        creators.add("1");
        for (SchoolTeacherDTO schoolTeacherDTO : schoolTeacherDTOs) {
            creators.add(schoolTeacherDTO.getUserId());
        }
        page.getParams().put("creators", creators);
        return listPageExcept(classInfoId, page);
    }

    @Override
    public SuccessResultList<List<QuestionDTO>> listPageExceptTeacher(String classInfoId, ListPage page) throws SearchException {
        List<String> creators = new ArrayList<>();
        // 列出超级管理员、当前老师的试题
        creators.add(securityComponent.getCurrentUser().getUserId());
        creators.add("1");
        page.getParams().put("creators", creators);
        return listPageExcept(classInfoId, page);
    }

    @Override
    public SuccessResultList<List<ClassQuestionDTO>> listPageOfMine(String token, String classInfoId, ListPage page) {
        page.getParams().put(KEY_IS_STUDENT, "isStudent");
        return classQuestionService.listPage(classInfoId, page);
    }

    @Override
    public UploadExcelResultDTO importExcel(MultipartFile excel) throws IOException {
        Map<String, Object> params = getHashMap(16);
        // 错误列表
        List<QuestionExcelError> questionExcelErrors = new ArrayList<>();
        long startTime = System.currentTimeMillis();
        LOG.debug("读取Excel");
        EasyExcel.read(excel.getInputStream(), QuestionExcel.class, new QuestionExcelListener() {
            @Override
            public void readList(List<QuestionExcel> questionExcels) {
                for (QuestionExcel questionExcel : questionExcels) {
                    try {
                        QuestionVO questionVO = new QuestionVO();
                        if (StringUtils.equals(QuestionExcelListener.QUESTION_CHOICE, questionExcel.getType())) {
                            questionVO.setType(QuestionTypeEnum.CHOICE.getValue());
                            if (StringUtils.equals(QuestionExcelListener.QUESTION_CHOICE_SINGLE, questionExcel.getKind())) {
                                questionVO.setChoiceType(QuestionChoiceTypeEnum.SINGLE.getValue());
                            } else if (StringUtils.equals(QuestionExcelListener.QUESTION_CHOICE_MULTIPLE, questionExcel.getKind())) {
                                questionVO.setChoiceType(QuestionChoiceTypeEnum.MULTIPLE.getValue());
                            } else {
                                throw new Exception("选择类型错误，只能是“单选”，“多选”");
                            }
                            if (questionExcel.getOptions().isEmpty()) {
                                throw new Exception("选择选项不能为空");
                            }
                        } else if (StringUtils.equals(QuestionExcelListener.QUESTION_TRUE_OR_FALSE, questionExcel.getType())) {
                            questionVO.setType(QuestionTypeEnum.TURE_OR_FALSE.getValue());
                        } else if (StringUtils.equals(QuestionExcelListener.QUESTION_FILL_IN_THE_BLANKS, questionExcel.getType())) {
                            questionVO.setType(QuestionTypeEnum.FILL_IN_THE_BLANKS.getValue());
                        } else if (StringUtils.equals(QuestionExcelListener.QUESTION_ANSWER, questionExcel.getType())) {
                            questionVO.setType(QuestionTypeEnum.ANSWER.getValue());
                        } else {
                            throw new Exception("选择类型错误，只能是“选择题”，“判断题”，“填空题”，“解答题”");
                        }

                        if (!StringUtils.equals(QuestionExcelListener.QUESTION_ANSWER, questionExcel.getType()) &&
                                !StringUtils.equals(QuestionExcelListener.QUESTION_FILL_IN_THE_BLANKS, questionExcel.getType())) {
                            if (questionExcel.getOptions().isEmpty()) {
                                throw new Exception("选项不能为空");
                            }
                            List<QuestionOptionsVO> questionOptions = new ArrayList<>();
                            for (int i = 0; i < questionExcel.getOptions().size(); i++) {
                                String optionContent = questionExcel.getOptions().get(i);
                                if (StringUtils.isBlank(optionContent)) {
                                    throw new Exception("选线有空值");
                                }
                                QuestionOptionsVO questionOptionsVO = new QuestionOptionsVO();
                                questionOptionsVO.setOption(Character.toString((char) (65 + i)));
                                questionOptionsVO.setContent(optionContent);
                                questionOptions.add(questionOptionsVO);
                            }
                            questionVO.setQuestionOptions(questionOptions);
                        }
                        questionVO.setAnalysis("<p>" + questionExcel.getAnalysis() + "<br></p>");
                        questionVO.setAnswer(questionExcel.getAnswer().replaceAll(",", "\\$_WenG_\\$").replaceAll("\\s", ""));
                        questionVO.setDifficulty(questionExcel.getDifficulty());
                        questionVO.setSubject("<p>" + questionExcel.getTitle() + "<br></p>");
                        saveQuestionInfo(null, questionVO);
                    } catch (Exception e) {
                        e.printStackTrace();
                        questionExcelErrors.add(getQuestionExcelError(questionExcel.getNo(), questionExcel.getTitle(), e.getMessage()));
                    }
                }
            }
        }).headRowNumber(2).sheet().doRead();
        long endTime = System.currentTimeMillis();
        String excelFileId = null;
        if (questionExcelErrors.size() > 0) {
            excelFileId = new AbstractErrorExcelHandler<QuestionExcelError>(fileService) {
                @Override
                public List<List<String>> excelHeaderNames() {
                    return simpleExcelHeader(new String[]{"序号", "标题", "错误原因"});
                }
            }.saveErrorExcel(questionExcelErrors);
        }
        return new UploadExcelResultDTO(questionExcelErrors.size(), endTime - startTime, excelFileId);
    }

    @Override
    public void updateCustomQuestionType(String questionTypeId, List<String> questionIds) {
        if (questionIds.isEmpty()) {
            return;
        }
        List<QuestionPO> questionPOs = listPOByIds(questionIds);
        Map<String, Object> params = getHashMap(4);
        for (QuestionPO questionPO : questionPOs) {
            String customQuestionType = questionPO.getCustomQuestionType();
            if (customQuestionType != null && customQuestionType.contains(questionTypeId)) {
                continue;
            }
            if (StringUtils.isBlank(customQuestionType)) {
                customQuestionType = questionTypeId;
            } else {
                customQuestionType += ("," + questionTypeId);
            }
            params.put("customQuestionType", customQuestionType);
            params.put("questionId", questionPO.getQuestionId());
            questionDao.updateCustomQuestionType(params);
        }
    }

    @Override
    public void removeCustomQuestionType(String questionTypeId, List<String> questionIds) {
        if (questionIds.isEmpty()) {
            return;
        }
        List<QuestionPO> questionPOs = listPOByIds(questionIds);
        Map<String, Object> params = getHashMap(4);
        for (QuestionPO questionPO : questionPOs) {
            String customQuestionType = questionPO.getCustomQuestionType();
            if (customQuestionType == null || !customQuestionType.contains(questionTypeId)) {
                continue;
            }
            customQuestionType = customQuestionType.replaceAll(questionTypeId, "");
            // 去除中间逗号
            customQuestionType = customQuestionType.replaceAll(",,", ",");
            // 去除第一个逗号
            if (customQuestionType.startsWith(",")) {
                customQuestionType = customQuestionType.substring(1);
            }
            // 去除最后逗号
            if (customQuestionType.endsWith(",")) {
                customQuestionType = customQuestionType.substring(0, customQuestionType.length() - 1);
            }
            params.put("customQuestionType", customQuestionType);
            params.put("questionId", questionPO.getQuestionId());
            questionDao.updateCustomQuestionType(params);
        }
    }

    private List<QuestionPO> listPOByIds(List<String> questionIds) {
        Map<String, Object> params = getHashMap(2);
        params.put("questionIds", questionIds);
        return questionDao.listPO(params);
    }

    /**
     * Excel错误信息
     *
     * @param no
     * @param title
     * @param reason
     * @return
     */
    private QuestionExcelError getQuestionExcelError(Integer no, String title, String reason) {
        QuestionExcelError questionExcelError = new QuestionExcelError();
        questionExcelError.setNo(no);
        questionExcelError.setTitle(title);
        questionExcelError.setReason(reason);
        return questionExcelError;
    }

}